/* -*- mode: js2; js2-basic-offset: 4; indent-tabs-mode: nil -*- */

const Lang = imports.lang;
const Clutter = imports.gi.Clutter;
const GLib = imports.gi.GLib;
const GObject = imports.gi.GObject;
const Atspi = imports.gi.Atspi;


const MOUSE = 'mouse';
const BUTTON_PRESSED = 'mouse:button:1p';
const MOUSE_MOVED = 'mouse:abs';
const BUTTON_RELEASED = 'mouse:button:1r';

const IDLE = 0;
const PROGRESS = 1;
const ACTIVATED = 2;
const CANCELED = 3;


const LongPressAction = new Lang.Class({
	Name: 'LongPressAction',
    Extends: Clutter.GestureAction,
    Signals: { 'activated': { param_types: [GObject.TYPE_INT, GObject.TYPE_INT] } },

    _init: function() {
        this.parent();
        
        this._threshold = Clutter.Settings.get_default().dndDragThreshold;
        this._duration = Clutter.Settings.get_default().longPressDuration;
        
        Atspi.init();
    	this._atspiListener = Atspi.EventListener.new(Lang.bind(this, this._onChanged));
        this._atspiListener.register(MOUSE);
        
        this._state = IDLE;
    },

    vfunc_gesture_prepare: function(action, actor) {
    	if (this._state == IDLE) {
    		this._device_type = this.get_last_event(0).get_source_device().get_device_type();
    		[this._startX, this._startY, mods] = global.get_pointer();
    		
    		this._state = PROGRESS;
    		return;
    	}
    	
    	this._state = CANCELED;
    },

    _onChanged: function(event) {
    	if (event.type == BUTTON_RELEASED) {
    		if (this._state == ACTIVATED)
        		this.emit('activated', this._startX, this._startY);
    		else if (this._timeoutId){
    			GLib.source_remove(this._timeoutId);
    			this._timeoutId = null;
    		}
    		
    		this._state = IDLE;
    	}
    	
    	if (this._device_type != Clutter.InputDeviceType.TOUCHSCREEN_DEVICE)
    		return;
    	
    	if (event.type == BUTTON_PRESSED) {
    		this._timeoutId = GLib.timeout_add(GLib.PRIORITY_DEFAULT, this._duration, Lang.bind(this, function() {
				this._timeoutId = null;
				if (this._state == PROGRESS) {
					this._state = ACTIVATED;
				}
			}));
    	}
    	else if (event.type == MOUSE_MOVED) {
	    	let [x, y, mods] = global.get_pointer();        	
	    	if (Math.abs(x - this._startX) > this._threshold &&
	    		Math.abs(y - this._startY) > this._threshold)
	    		this._state = CANCELED;
    	}
	}
});


function enable() {
	let longPressAction = new LongPressAction();
	longPressAction.connect('activated', function(action, x, y){
		Atspi.generate_mouse_event(x, y, 'b3c');
	});
	global.stage.add_action(longPressAction);
}

function disable() {
	global.stage.remove_action_by_name('LongPressAction');
}
